'use strict';

var assign = require('object-assign');
var React = require('react');
var classnames = require('classnames');
var ReactTransitionEvents = require('react/lib/ReactTransitionEvents');

var helpers = {
  initialize: function initialize(props) {
    var slideCount = React.Children.count(props.children);
    var listWidth = this.refs.list.getDOMNode().getBoundingClientRect().width;
    var trackWidth = this.refs.track.getDOMNode().getBoundingClientRect().width;
    var slideWidth = this.getDOMNode().getBoundingClientRect().width / props.slidesToShow;

    this.setState({
      slideCount: slideCount,
      slideWidth: slideWidth,
      listWidth: listWidth,
      trackWidth: trackWidth,
      currentSlide: props.initialSlide

    }, function () {
      // getCSS function needs previously set state
      var trackStyle = this.getCSS(this.getLeft(this.state.currentSlide));
      this.setState({ trackStyle: trackStyle });

      this.autoPlay(); // once we're set up, trigger the initial autoplay.
    });
  },
  getDotCount: function getDotCount() {
    var pagerQty;
    pagerQty = Math.ceil(this.state.slideCount / this.props.slidesToScroll);
    return pagerQty - 1;
  },
  getLeft: function getLeft(slideIndex) {
    var slideOffset = 0;
    var targetLeft;
    var targetSlide;
    if (this.props.infinite === true) {
      if (this.state.slideCount > this.props.slidesToShow) {
        slideOffset = this.state.slideWidth * this.props.slidesToShow * -1;
      }
      if (this.state.slideCount % this.props.slidesToScroll !== 0) {
        if (slideIndex + this.props.slidesToScroll > this.state.slideCount && this.state.slideCount > this.props.slidesToShow) {
          if (slideIndex > this.state.slideCount) {
            slideOffset = (this.props.slidesToShow - (slideIndex - this.state.slideCount)) * this.state.slideWidth * -1;
          } else {
            slideOffset = this.state.slideCount % this.props.slidesToScroll * this.state.slideWidth * -1;
          }
        }
      }
    } else {}
    if (this.props.centerMode === true && this.props.infinite === true) {
      slideOffset += this.state.slideWidth * Math.floor(this.props.slidesToShow / 2) - this.state.slideWidth;
    } else if (this.props.centerMode === true) {
      slideOffset = this.state.slideWidth * Math.floor(this.props.slidesToShow / 2);
    }

    targetLeft = slideIndex * this.state.slideWidth * -1 + slideOffset;

    if (this.props.variableWidth === true) {
      var targetSlideIndex;
      if (this.state.slideCount <= this.props.slidesToShow || this.props.infinite === false) {
        targetSlide = this.refs.track.getDOMNode().childNodes[slideIndex];
      } else {
        targetSlideIndex = slideIndex + this.props.slidesToShow;
        targetSlide = this.refs.track.getDOMNode().childNodes[targetSlideIndex];
      }
      targetLeft = targetSlide ? targetSlide.offsetLeft * -1 : 0;
      if (this.props.centerMode === true) {
        if (this.props.infinite === false) {
          targetSlide = this.refs.track.getDOMNode().childNodes[slideIndex];
        } else {
          targetSlide = this.refs.track.getDOMNode().childNodes[slideIndex + this.props.slidesToShow + 1];
        }

        targetLeft = targetSlide ? targetSlide.offsetLeft * -1 : 0;
        targetLeft += (this.state.listWidth - targetSlide.offsetWidth) / 2;
      }
    }

    return targetLeft;
  },
  getAnimateCSS: function getAnimateCSS(targetLeft) {
    var style = this.getCSS(targetLeft);
    style.WebkitTransition = '-webkit-transform ' + this.props.speed + 'ms ' + this.props.cssEase;
    style.transition = 'transform ' + this.props.speed + 'ms ' + this.props.cssEase;
    return style;
  },
  getCSS: function getCSS(targetLeft) {
    // implemented this instead of setCSS
    var trackWidth;
    if (this.props.variableWidth) {
      trackWidth = (this.state.slideCount + 2 * this.props.slidesToShow) * this.state.slideWidth;
    } else if (this.props.centerMode) {
      trackWidth = (this.state.slideCount + 2 * (this.props.slidesToShow + 1)) * this.state.slideWidth;
    } else {
      trackWidth = (this.state.slideCount + 2 * this.props.slidesToShow) * this.state.slideWidth;
    }
    var style = {
      opacity: 1,
      width: trackWidth,
      WebkitTransform: 'translate3d(' + targetLeft + 'px, 0px, 0px)',
      transform: 'translate3d(' + targetLeft + 'px, 0px, 0px)',
      transition: '',
      WebkitTransition: ''
    };

    return style;
  },
  getSlideStyle: function getSlideStyle() {
    return {
      width: this.state.slideWidth
    };
  },
  getSlideClasses: function getSlideClasses(index) {
    var slickActive, slickCenter, slickCloned;
    var currentSlide = this.state.currentSlide;
    var centerOffset;

    slickCloned = index < 0 || index >= this.state.slideCount;
    if (this.props.centerMode) {
      centerOffset = Math.floor(this.props.slidesToShow / 2);
      slickCenter = currentSlide === index;
      if (index > currentSlide - centerOffset - 1 && index <= currentSlide + centerOffset) {
        slickActive = true;
      }
    } else {
      slickActive = currentSlide === index;
    }
    return classnames({
      'slick-slide': true,
      'slick-active': slickActive,
      'slick-center': slickCenter,
      'slick-cloned': slickCloned
    });
  },
  adaptHeight: function adaptHeight() {
    if (this.props.adaptiveHeight) {
      var selector = '[data-index="' + this.state.currentSlide + '"]';
      if (this.refs.list) {
        var slickList = this.refs.list.getDOMNode();
        slickList.style.height = slickList.querySelector(selector).offsetHeight + 'px';
      }
    }
  },
  slideHandler: function slideHandler(index, sync, dontAnimate) {
    // Functionality of animateSlide and postSlide is merged into this function
    // console.log('slideHandler', index);
    var targetSlide, currentSlide;
    var targetLeft, currentLeft;

    if (this.state.animating === true) {
      return;
    }

    // To prevent the slider from sticking in animating state, If we click on already active dot
    if (this.props.fade === true && this.state.currentSlide === index) {
      return;
    }
    if (this.state.slideCount <= this.props.slidesToShow) {
      return;
    }

    targetSlide = index;
    if (targetSlide < 0) {
      if (this.state.slideCount % this.props.slidesToScroll !== 0) {
        currentSlide = this.state.slideCount - this.state.slideCount % this.props.slidesToScroll;
      } else {
        currentSlide = this.state.slideCount + targetSlide;
      }
    } else if (targetSlide >= this.state.slideCount) {
      if (this.state.slideCount % this.props.slidesToScroll !== 0) {
        currentSlide = 0;
      } else {
        currentSlide = targetSlide - this.state.slideCount;
      }
    } else {
      currentSlide = targetSlide;
    }

    targetLeft = this.getLeft(targetSlide, this.state);
    currentLeft = this.getLeft(currentSlide, this.state);

    if (this.props.infinite === false) {
      targetLeft = currentLeft;
    }

    if (this.props.afterChange !== null) {
      this.props.afterChange(currentSlide);
    }

    var callback = (function () {
      this.setState({
        animating: false,
        trackStyle: this.getCSS(currentLeft),
        swipeLeft: null
      });
    }).bind(this);

    this.setState({
      animating: true,
      currentSlide: currentSlide,
      currentLeft: currentLeft,
      trackStyle: this.getAnimateCSS(targetLeft)
    }, function () {
      ReactTransitionEvents.addEndEventListener(this.refs.track.getDOMNode(), callback);
      setTimeout(callback, this.props.speed);
    });

    this.autoPlay();
  },
  swipeDirection: function swipeDirection(touchObject) {
    var xDist, yDist, r, swipeAngle;

    xDist = touchObject.startX - touchObject.curX;
    yDist = touchObject.startY - touchObject.curY;
    r = Math.atan2(yDist, xDist);

    swipeAngle = Math.round(r * 180 / Math.PI);
    if (swipeAngle < 0) {
      swipeAngle = 360 - Math.abs(swipeAngle);
    }
    if (swipeAngle <= 45 && swipeAngle >= 0) {
      return this.props.rtl === false ? 'left' : 'right';
    }
    if (swipeAngle <= 360 && swipeAngle >= 315) {
      return this.props.rtl === false ? 'left' : 'right';
    }
    if (swipeAngle >= 135 && swipeAngle <= 225) {
      return this.props.rtl === false ? 'right' : 'left';
    }

    return 'vertical';
  },
  autoPlay: function autoPlay() {
    var play = (function () {
      if (this.isMounted()) {
        this.slideHandler(this.state.currentSlide + this.props.slidesToScroll);
      }
    }).bind(this);
    if (this.props.autoplay) {
      window.clearTimeout(this.state.autoPlayTimer);
      this.setState({
        autoPlayTimer: window.setTimeout(play, this.props.autoplaySpeed)
      });
    }
  }
};

module.exports = helpers;